@@ -1,4 +1,4 @@
-__version__ = '2.6.1'
+__version__ = '2.8.12'
 
 from tri_declarative import LAST
 
@@ -7,6 +7,7 @@ from iommi._db_compat import (
     setup_db_compat,
 )
 from iommi.action import Action
+from iommi.asset import Asset
 from iommi.base import MISSING
 from iommi.form import (
     Field,
@@ -18,6 +19,9 @@ from iommi.fragment import (
     Header,
     html,
 )
+from iommi.from_model import (
+    register_search_fields,
+)
 from iommi.menu import (
     Menu,
     MenuItem,
@@ -41,10 +45,6 @@ from iommi.table import (
     register_column_factory,
     Table,
 )
-from iommi.from_model import (
-    register_search_fields,
-)
-from iommi.asset import Asset
 
 setup_db_compat()
 
@@ -70,8 +70,10 @@ __all__ = [
     'Asset',
     'Column',
     'Field',
+    'Fragment',
     'Form',
     'Fragment',
+    'Header',
     'Menu',
     'MenuItem',
     'middleware',
@@ -81,6 +83,7 @@ __all__ = [
     'Table',
     'Filter',
     'LAST',
+    'MISSING',
     'register_factory',
     'register_field_factory',
     'register_filter_factory',
@@ -92,4 +95,4 @@ __all__ = [
     'html',
 ]
 
-default_app_config = 'iommi.django_app.IommiConfig'
+default_app_config = 'iommi.apps.IommiConfig'
