@@ -16,6 +16,36 @@ from django.db.models import (
     QuerySet,
 )
 from django.utils.translation import gettext
+from pyparsing import (
+    alphanums,
+    alphas,
+    Char,
+    Forward,
+    Group,
+    Keyword,
+    oneOf,
+    ParseException,
+    ParseResults,
+    QuotedString,
+    quotedString,
+    Word,
+    ZeroOrMore,
+)
+from tri_declarative import (
+    class_shortcut,
+    declarative,
+    dispatch,
+    EMPTY,
+    flatten,
+    getattr_path,
+    Namespace,
+    Refinable,
+    refinable,
+    setdefaults_path,
+    Shortcut,
+    with_meta,
+)
+from tri_struct import Struct
 
 from iommi import (
     Action,
@@ -31,6 +61,7 @@ from iommi.base import (
     keys,
     MISSING,
     model_and_rows,
+    NOT_BOUND_MESSAGE,
     values,
 )
 from iommi.debug import iommi_debug_on
@@ -61,46 +92,16 @@ from iommi.part import (
     Part,
     request_data,
 )
-from iommi.traversable import (
-    declared_members,
-    EvaluatedRefinable,
-    set_declared_member,
-)
 from iommi.reinvokable import (
     reinvokable,
     reinvoke,
     set_and_remember_for_reinvoke,
 )
-from pyparsing import (
-    alphanums,
-    alphas,
-    Char,
-    Forward,
-    Group,
-    Keyword,
-    oneOf,
-    ParseException,
-    ParseResults,
-    QuotedString,
-    quotedString,
-    Word,
-    ZeroOrMore,
+from iommi.traversable import (
+    declared_members,
+    EvaluatedRefinable,
+    set_declared_member,
 )
-from tri_declarative import (
-    class_shortcut,
-    declarative,
-    dispatch,
-    EMPTY,
-    getattr_path,
-    Namespace,
-    Refinable,
-    refinable,
-    setdefaults_path,
-    Shortcut,
-    with_meta,
-    flatten
-)
-from tri_struct import Struct
 
 
 class QueryException(Exception):
@@ -124,7 +125,7 @@ Q_OPERATOR_BY_QUERY_OPERATOR = {
     ':': 'icontains',
 }
 
-FREETEXT_SEARCH_NAME = 'freetext'
+FREETEXT_SEARCH_NAME = 'freetext_search'
 
 _filter_factory_by_django_field_type = {}
 
@@ -149,23 +150,25 @@ def value_to_str_for_query(filter, v):
         return str(v)
     if isinstance(v, Model):
         model = type(v)
-        search_field = filter.search_fields[0]
+        search_field = filter.search_fields[-1]
         try:
             v = getattr_path(v, search_field)
         except AttributeError:
-            raise NoRegisteredSearchFieldException(f'{model.__name__} has no attribute {search_field}. Please register search fields with register_search_fields or specify search_fields.')
+            raise NoRegisteredSearchFieldException(
+                f'{model.__name__} has no attribute {search_field}. Please register search fields with register_search_fields or specify search_fields.'
+            )
     return to_string_surrounded_by_quote(v)
 
 
-def build_query_expression(*, field, filter, value):
+def build_query_expression(*, filter, value):
     if isinstance(value, Model):
         try:
             # We ignore the return value on purpose here. We are after the raise.
             get_search_fields(model=type(value))
         except NoRegisteredSearchFieldException:
-            return f'{field._name}.pk={value.pk}'
+            return f'{filter.query_name}.pk={value.pk}'
 
-    return f'{field._name}{filter.query_operator_for_field}{value_to_str_for_query(filter, value)}'
+    return f'{filter.query_name}{filter.query_operator_for_field}{value_to_str_for_query(filter, value)}'
 
 
 def case_sensitive_query_operator_to_q_operator(op):
@@ -203,11 +206,17 @@ choice_queryset_value_to_q.iommi_needs_attr = True
 
 
 def default_filter__is_valid_filter(name, filter, **_):
-    return filter.attr or filter.value_to_q, f"{name} cannot be a part of a query, it has no attr or value_to_q so we don't know what to search for. If you want to include it anyway you can define the callback is_valid_filter which should return a boolean and a string with an error message if the boolean is False. The simplest way to do that would be is_valid_filter=lambda **_: (True, '') (filter__is_valid_filter=lambda **_: (True, '') for a Column)"
+    return (
+        filter.attr or filter.value_to_q,
+        f"{name} cannot be a part of a query, it has no attr or value_to_q so we don't know what to search for. If you want to include it anyway you can define the callback is_valid_filter which should return a boolean and a string with an error message if the boolean is False. The simplest way to do that would be is_valid_filter=lambda **_: (True, '') (filter__is_valid_filter=lambda **_: (True, '') for a Column)",
+    )
 
 
 def choice_queryset__is_valid_filter(name, filter, **_):
-    return filter.attr, f"{name} cannot be a part of a query, it has no attr so we don't know what to search for. If you want to include it anyway you can define the callback is_valid_filter which should return a boolean and a string with an error message if the boolean is False. The simplest way to do that would be is_valid_filter=lambda **_: (True, '') (filter__is_valid_filter=lambda **_: (True, '') for a Column)"
+    return (
+        filter.attr,
+        f"{name} cannot be a part of a query, it has no attr so we don't know what to search for. If you want to include it anyway you can define the callback is_valid_filter which should return a boolean and a string with an error message if the boolean is False. The simplest way to do that would be is_valid_filter=lambda **_: (True, '') (filter__is_valid_filter=lambda **_: (True, '') for a Column)",
+    )
 
 
 @with_meta
@@ -229,6 +238,7 @@ class Filter(Part):
     search_fields = Refinable()
     unary = Refinable()
     is_valid_filter = Refinable()
+    query_name = EvaluatedRefinable()
 
     @reinvokable
     @dispatch(
@@ -237,8 +247,11 @@ class Filter(Part):
         search_fields=MISSING,
         field__required=False,
         # TODO: this isn't right, freetext can be a callable
-        field__include=lambda query, field, **_: not query.filters._declared_members.get(field._name, Struct(freetext=False)).freetext,
+        field__include=lambda query, field, **_: not query.filters._declared_members.get(
+            field._name, Struct(freetext=False)
+        ).freetext,
         is_valid_filter=default_filter__is_valid_filter,
+        query_name=lambda filter, **_: filter.iommi_name(),
     )
     def __init__(self, **kwargs):
         """
@@ -247,6 +260,9 @@ class Filter(Part):
         :param field__include: set to `True` to display a GUI element for this filter in the basic style interface.
         :param field__call_target: the factory to create a `Field` for the basic GUI, for example `Field.choice`. Default: `Field`
         """
+        model_field = kwargs.get('model_field')
+        if model_field and model_field.remote_field:
+            kwargs['model'] = model_field.remote_field.model
 
         super(Filter, self).__init__(**kwargs)
 
@@ -262,8 +278,6 @@ class Filter(Part):
                 self.search_fields = get_search_fields(model=self.model)
             except NoRegisteredSearchFieldException:
                 self.search_fields = ['pk']
-                if iommi_debug_on():
-                    print(f'Warning: falling back to primary key as lookup and sorting on {self._name}. \nTo get rid of this warning and get a nicer lookup and sorting use register_search_fields for model {self.model}')
 
     def own_evaluate_parameters(self):
         return dict(filter=self)
@@ -307,8 +321,9 @@ class Filter(Part):
             factory_lookup=_filter_factory_by_django_field_type,
             model_field_name=model_field_name,
             model_field=model_field,
-            defaults_factory=lambda model_field: {},
-            **kwargs)
+            defaults_factory=lambda model_field: {},  # TODO: this is wrong! but base_defaults_factory doesn't work either.. there's no display_name on Filter
+            **kwargs,
+        )
 
     @classmethod
     @class_shortcut(
@@ -334,9 +349,13 @@ class Filter(Part):
         Field that has one value out of a set.
         :type choices: list
         """
-        setdefaults_path(kwargs, dict(
-            field__choices=kwargs.get('choices'),
-        ))
+        assert 'choices' in kwargs, 'To use Filter.choice, you must pass the choices list'
+        setdefaults_path(
+            kwargs,
+            dict(
+                field__choices=kwargs.get('choices'),
+            ),
+        )
         return call_target(**kwargs)
 
     @classmethod
@@ -348,9 +367,12 @@ class Filter(Part):
         Field that has one value out of a set.
         :type choices: list
         """
-        setdefaults_path(kwargs, dict(
-            field__choices=kwargs.get('choices'),
-        ))
+        setdefaults_path(
+            kwargs,
+            dict(
+                field__choices=kwargs.get('choices'),
+            ),
+        )
         return call_target(**kwargs)
 
     @classmethod
@@ -365,14 +387,19 @@ class Filter(Part):
         Field that has one value out of a set.
         """
         if 'model' not in kwargs:
-            assert isinstance(choices, QuerySet), 'The convenience feature to automatically get the parameter model set only works for QuerySet instances'
+            assert isinstance(
+                choices, QuerySet
+            ), 'The convenience feature to automatically get the parameter model set only works for QuerySet instances'
             kwargs['model'] = choices.model
 
-        setdefaults_path(kwargs, dict(
-            field__choices=choices,
-            field__model=kwargs['model'],
-            choices=choices,
-        ))
+        setdefaults_path(
+            kwargs,
+            dict(
+                field__choices=choices,
+                field__model=kwargs['model'],
+                choices=choices,
+            ),
+        )
         return call_target(**kwargs)
 
     @classmethod
@@ -494,7 +521,6 @@ class Filter(Part):
             choices=model_field.remote_field.model.objects.all(),
             extra__django_related_field=True,
         )
-        kwargs['model'] = model_field.remote_field.model
         return call_target(model_field=model_field, **kwargs)
 
 
@@ -526,9 +552,7 @@ class QueryAutoConfig(AutoConfig):
 class Advanced(Fragment):
     toggle: Namespace = Refinable()
 
-    @dispatch(
-        toggle=EMPTY
-    )
+    @dispatch(toggle=EMPTY)
     @reinvokable
     def __init__(self, **kwargs):
         super(Advanced, self).__init__(**kwargs)
@@ -536,12 +560,11 @@ class Advanced(Fragment):
         toggle = setdefaults_path(
             Namespace(),
             self.toggle,
+            _name='toggle',
             call_target=Action,
             attrs__href='#',
             attrs__class__iommi_query_toggle_simple_mode=True,
-            attrs={
-                'data-advanced-mode': 'simple'
-            },
+            attrs={'data-advanced-mode': 'simple'},
             display_name=gettext('Switch to advanced search'),
         )
         self.toggle = declared_members(self).toggle = toggle()
@@ -606,8 +629,10 @@ class Query(Part):
                 exclude=auto.exclude,
             )
 
-            assert model is None, "You can't use the auto feature and explicitly pass model. " \
-                                  "Either pass auto__model, or we will set the model for you from auto__rows"
+            assert model is None, (
+                "You can't use the auto feature and explicitly pass model. "
+                "Either pass auto__model, or we will set the model for you from auto__rows"
+            )
             model = auto_model
 
             if rows is None:
@@ -634,11 +659,7 @@ class Query(Part):
             kwargs = Namespace(flatten(Namespace(kwargs)))
             freetext_config = kwargs.get('form', {}).get('fields', {}).pop('freetext', {})
 
-        super(Query, self).__init__(
-            model=model,
-            rows=rows,
-            **kwargs
-        )
+        super(Query, self).__init__(model=model, rows=rows, **kwargs)
 
         collect_members(self, name='filters', items=filters, items_dict=_filters_dict, cls=self.get_meta().member_class)
 
@@ -651,7 +672,7 @@ class Query(Part):
             required=False,
             include=False,
             help__include=False,
-            **freetext_config
+            **freetext_config,
         )
 
         for name, filter in items(declared_members(self).filters):
@@ -678,7 +699,7 @@ class Query(Part):
         )
         declared_members(self).form = self.form
 
-        self.advanced = declared_members(self).advanced = self.advanced()
+        self.advanced = declared_members(self).advanced = self.advanced(_name='advanced')
 
         self.form_container = self.form_container(_name='form_container')
 
@@ -701,6 +722,10 @@ class Query(Part):
         return render(request=self.get_request())
 
     def on_bind(self) -> None:
+        # Prevent the nested form from thinking it's a part of a nested form set up
+        if 'form' in self._evaluate_parameters:
+            del self._evaluate_parameters['form']
+
         bind_members(self, name='filters')
 
         request = self.get_request()
@@ -743,6 +768,8 @@ class Query(Part):
 
         self.form_container = self.form_container.bind(parent=self)
 
+        self.filter_name_by_query_name = {x.query_name: name for name, x in items(self.filters)}
+
     @staticmethod
     @refinable
     def filter(query, rows, **_):
@@ -755,10 +782,7 @@ class Query(Part):
             if q:
                 rows = rows.filter(q)
 
-        return query.postprocess(
-            rows=rows,
-            **query.iommi_evaluate_parameters()
-        )
+        return query.postprocess(rows=rows, **query.iommi_evaluate_parameters())
 
     @staticmethod
     @refinable
@@ -772,14 +796,14 @@ class Query(Part):
         return '-' + path_join(self.iommi_path, 'query')
 
     def parse_query_string(self, query_string: str) -> Q:
-        assert self._is_bound
+        assert self._is_bound, NOT_BOUND_MESSAGE
         query_string = query_string.strip()
         if not query_string:
             return Q()
         parser = self._create_grammar()
         try:
             tokens = parser.parseString(query_string, parseAll=True)
-        except ParseException as e:
+        except ParseException:
             raise QueryException('Invalid syntax for query')
         return self._compile(tokens)
 
@@ -851,7 +875,9 @@ class Query(Part):
         something < 10 AND other >= 2015-01-01 AND (foo < 1 OR bar > 1)
 
         """
-        quoted_string_excluding_quotes = QuotedString('"', escChar='\\').setParseAction(lambda token: StringValue(token[0]))
+        quoted_string_excluding_quotes = QuotedString('"', escChar='\\').setParseAction(
+            lambda token: StringValue(token[0])
+        )
         and_ = Keyword('and', caseless=True)
         or_ = Keyword('or', caseless=True)
         binary_op = oneOf('=> =< = < > >= <= : != !:', caseless=True).setResultsName('operator')
@@ -889,20 +915,40 @@ class Query(Part):
         filter = self.filters.get(filter_name.lower())
         if filter:
             if not filter.unary:
-                raise QueryException(f'"{filter_name}" is not a unary filter, you must use it like "{filter_name}=something"')
+                raise QueryException(
+                    f'"{filter_name}" is not a unary filter, you must use it like "{filter_name}=something"'
+                )
             result = filter.value_to_q(filter=filter, op='=', value_string_or_f=value)
             return result
-        raise QueryException(f'Unknown unary filter "{filter_name}", available filters: {", ".join(list(keys(self.filters)))}')
+        raise QueryException(
+            f'Unknown unary filter "{filter_name}", available filters: {", ".join(list(keys(self.filters)))}'
+        )
 
     def _binary_op_to_q(self, token):
         """
         Convert a parsed token of filter_name OPERATOR filter_name into a Q object
         """
-        assert self._is_bound
-        filter_name, op, value_string_or_filter_name = token
+        assert self._is_bound, NOT_BOUND_MESSAGE
+        query_name, op, value_string_or_filter_name = token
 
-        if filter_name.endswith('.pk'):
-            filter = self.filters.get(filter_name.lower()[:-len('.pk')])
+        pk_lookup = False
+        if query_name.endswith('.pk'):
+            query_name = query_name[: -len('.pk')]
+            pk_lookup = True
+
+        filter_name = self.filter_name_by_query_name.get(query_name)
+        if filter_name is None:
+            raise QueryException(
+                f'Unknown filter "{query_name}", available filters: {list(keys(self.filter_name_by_query_name))}'
+            )
+
+        filter = self.filters.get(filter_name.lower())
+        if filter is None:
+            raise QueryException(
+                f'Unknown filter "{query_name}", available filters: {list(keys(self.filter_name_by_query_name))}'
+            )
+
+        if pk_lookup:
             if op != '=':
                 raise QueryException('Only = is supported for primary key lookup')
 
@@ -913,20 +959,21 @@ class Query(Part):
 
             return Q(**{f'{filter.attr}__pk': pk})
 
-        filter = self.filters.get(filter_name.lower())
-        if filter:
-            if isinstance(value_string_or_filter_name, str) and not isinstance(value_string_or_filter_name, StringValue) and value_string_or_filter_name.lower() in self.filters:
-                value_string_or_f = F(self.filters[value_string_or_filter_name.lower()].attr)
-            else:
-                value_string_or_f = value_string_or_filter_name
-            try:
-                result = filter.value_to_q(filter=filter, op=op, value_string_or_f=value_string_or_f)
-            except ValidationError as e:
-                raise QueryException(f'{e.message}')
-            if result is None:
-                raise QueryException(f'Unknown value "{value_string_or_f}" for filter "{filter._name}"')
-            return result
-        raise QueryException(f'Unknown filter "{filter_name}", available filters: {list(keys(self.filters))}')
+        if (
+            isinstance(value_string_or_filter_name, str)
+            and not isinstance(value_string_or_filter_name, StringValue)
+            and value_string_or_filter_name.lower() in self.filters
+        ):
+            value_string_or_f = F(self.filters[value_string_or_filter_name.lower()].attr)
+        else:
+            value_string_or_f = value_string_or_filter_name
+        try:
+            result = filter.value_to_q(filter=filter, op=op, value_string_or_f=value_string_or_f)
+        except ValidationError as e:
+            raise QueryException(f'{e.message}')
+        if result is None:
+            raise QueryException(f'Unknown value "{value_string_or_f}" for filter "{query_name}"')
+        return result
 
     def _freetext_to_q(self, token):
         if all(not v.freetext for v in values(self.filters)):
@@ -940,7 +987,7 @@ class Query(Part):
                 Q(**{filter.attr + '__' + filter.query_operator_to_q_operator(':'): token})
                 for filter in values(self.filters)
                 if filter.freetext
-            ]
+            ],
         )
 
     def get_query_string(self):
@@ -956,25 +1003,31 @@ class Query(Part):
         if request_data(request).get(self.get_advanced_query_param(), '').strip():
             return request_data(request).get(self.get_advanced_query_param())
         elif form.is_valid():
+
             def expr(field, is_list, value):
                 if is_list:
                     return '(' + ' OR '.join([expr(field, is_list=False, value=x) for x in field.value]) + ')'
-                return build_query_expression(field=field, filter=self.filters[field._name], value=value)
+                return build_query_expression(filter=self.filters[field._name], value=value)
 
             result = [
                 expr(field, field.is_list, field.value)
                 for field in values(form.fields)
-                if field._name != FREETEXT_SEARCH_NAME and field._name in self.filters and field.value not in (None, '', [])
+                if field._name != FREETEXT_SEARCH_NAME
+                and field._name in self.filters
+                and field.value not in (None, '', [])
             ]
 
             if FREETEXT_SEARCH_NAME in form.fields:
                 freetext = form.fields[FREETEXT_SEARCH_NAME].value
                 if freetext:
                     result.append(
-                        '(%s)' % ' or '.join([
-                            f'{filter._name}:{to_string_surrounded_by_quote(freetext)}'
-                            for filter in values(self.filters)
-                            if filter.freetext]
+                        '(%s)'
+                        % ' or '.join(
+                            [
+                                f'{filter.query_name}:{to_string_surrounded_by_quote(freetext)}'
+                                for filter in values(self.filters)
+                                if filter.freetext
+                            ]
                         )
                     )
             return ' and '.join(result)
@@ -997,9 +1050,7 @@ class Query(Part):
     )
     def filters_from_model(cls, filters, **kwargs):
         return create_members_from_model(
-            member_class=cls.get_meta().member_class,
-            member_params_by_member_name=filters,
-            **kwargs
+            member_class=cls.get_meta().member_class, member_params_by_member_name=filters, **kwargs
         )
 
     @classmethod
@@ -1007,10 +1058,11 @@ class Query(Part):
         filters=EMPTY,
     )
     def _from_model(cls, *, rows=None, model=None, filters, include=None, exclude=None):
-        assert rows is None or isinstance(rows, QuerySet), \
-            'auto__rows needs to be a QuerySet for filter generation to work. ' \
-            'If it needs to be a lambda, provide a model with auto__model for filter generation, ' \
+        assert rows is None or isinstance(rows, QuerySet), (
+            'auto__rows needs to be a QuerySet for filter generation to work. '
+            'If it needs to be a lambda, provide a model with auto__model for filter generation, '
             'and pass the lambda as rows.'
+        )
 
         model, rows = model_and_rows(model, rows)
         assert model is not None or rows is not None, "auto__model or auto__rows must be specified"
