@@ -14,6 +14,7 @@ from tri_declarative import (
     getattr_path,
     Namespace,
     RefinableObject,
+    setdefaults_path,
 )
 
 from iommi.base import (
@@ -74,10 +75,13 @@ def recursive_namespace(d):
 
 class Style:
     @dispatch(
-        assets=EMPTY,
+        root=EMPTY,
     )
-    def __init__(self, *bases, base_template=None, content_block=None, assets=None, **kwargs):
+    def __init__(
+        self, *bases, base_template=None, content_block=None, assets=None, root=None, internal=False, **kwargs
+    ):
         self.name = None
+        self.internal = internal
 
         self.base_template = base_template
         if not self.base_template:
@@ -93,7 +97,17 @@ class Style:
                     self.content_block = base.content_block
                     break
 
-        self.assets = {k: v for k, v in Namespace(*(base.assets for base in bases), assets).items() if v is not None}
+        if assets:
+            from iommi.debug import iommi_debug_on
+
+            if iommi_debug_on():
+                print(
+                    "Warning: The preferred way to add top level assets config to a Style is via the root argument. "
+                    "I.e. assets__* becomes root__assets__*"
+                )
+            setdefaults_path(root, assets=assets)
+
+        self.root = {k: v for k, v in items(Namespace(*(base.root for base in bases), root)) if v is not None}
         self.config = Namespace(*[x.config for x in bases], recursive_namespace(kwargs))
 
     def component(self, obj, is_root=False):
@@ -104,9 +118,6 @@ class Style:
         """
         result = Namespace()
 
-        if self.assets and is_root:
-            result.assets = self.assets
-
         # TODO: is this wrong? Should it take classes first, then loop through shortcuts?
         for class_name in class_names_for(type(obj)):
             if class_name in self.config:
@@ -116,6 +127,10 @@ class Style:
 
                 for shortcut_name in reversed(getattr(obj, '__tri_declarative_shortcut_stack', [])):
                     result = Namespace(result, shortcuts_config.get(shortcut_name, {}))
+
+        if is_root:
+            result = Namespace(result, self.root)
+
         return result
 
 
@@ -123,7 +138,7 @@ _styles = {}
 
 
 def register_style(name, conf):
-    assert name not in _styles
+    assert name not in _styles, f'{name} is already registered'
     assert conf.name is None
     conf.name = name
     _styles[name] = conf
@@ -139,15 +154,18 @@ def get_style(name):
         return _styles[name]
     except KeyError:
         style_names = "\n    ".join(_styles.keys())
-        raise Exception(f'''No registered style {name}. Register a style with register_style(). 
+        raise Exception(
+            f'''No registered style {name}. Register a style with register_style().
 
 Available styles:
-    {style_names}''') from None
+    {style_names}'''
+        ) from None
 
 
 def reinvoke_new_defaults(obj: Any, additional_kwargs: Dict[str, Any]) -> Any:
-    assert is_reinvokable(obj), f'reinvoke_new_defaults() called on object with ' \
-                                f'missing @reinvokable constructor decorator: {obj!r}'
+    assert is_reinvokable(obj), (
+        f'reinvoke_new_defaults() called on object with ' f'missing @reinvokable constructor decorator: {obj!r}'
+    )
     additional_kwargs_namespace = Namespace(additional_kwargs)
 
     kwargs = Namespace(additional_kwargs_namespace)
@@ -166,10 +184,14 @@ def reinvoke_new_defaults(obj: Any, additional_kwargs: Dict[str, Any]) -> Any:
                 else:
                     kwargs[name] = saved_param
 
-    additional_kwargs_namespace.pop('call_target', None)
-
     try:
-        result = type(obj)(**kwargs)
+        call_target = kwargs.pop('call_target', None)
+        if call_target is not None:
+            kwargs['call_target'] = Namespace(call_target, cls=type(obj))
+        else:
+            kwargs['call_target'] = type(obj)
+
+        result = kwargs()
     except TypeError as e:
         raise InvalidStyleConfigurationException(
             f'Object {obj!r} could not be updated with style configuration {flatten(additional_kwargs_namespace)}'
@@ -216,18 +238,20 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
         from iommi.table import Paginator
         from iommi.menu import (
             MenuBase,
-            DebugMenu,
+            get_debug_menu,
         )
         from iommi.error import Errors
         from iommi.action import Actions
         from iommi.admin import Admin
         from iommi.fragment import Container
         from iommi.fragment import Header
+        from iommi.live_edit import LiveEditPage
+
         default_classes = [
             Action,
             Actions,
             Column,
-            DebugMenu,
+            get_debug_menu().__class__,
             Errors,
             Field,
             Form,
@@ -241,6 +265,7 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
             Admin,
             Container,
             Header,
+            LiveEditPage,
         ]
     if additional_classes is None:
         additional_classes = []
@@ -278,8 +303,7 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
 
     if invalid_class_names or non_existent_shortcut_names:
         invalid_class_names_str = '\n'.join(
-            f'    Style: {style_name} - class: {cls_name}'
-            for style_name, cls_name in invalid_class_names
+            f'    Style: {style_name} - class: {cls_name}' for style_name, cls_name in invalid_class_names
         )
         if invalid_class_names_str:
             invalid_class_names_str = 'Invalid class names:\n' + invalid_class_names_str
