@@ -1,13 +1,13 @@
+from iommi.asset import Asset
 from iommi.style import (
     Style,
 )
-from iommi.style_font_awesome_4 import font_awesome_4
 from iommi.style_base import base
-from iommi.asset import Asset
+from iommi.style_font_awesome_4 import font_awesome_4
 
 bootstrap_base = Style(
     base,
-    assets=dict(
+    root__assets=dict(
         css=Asset.css(
             attrs=dict(
                 href='https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/css/bootstrap.min.css',
@@ -68,6 +68,7 @@ bootstrap_base = Style(
             'form-text': True,
             'text-muted': True,
         },
+        non_editable_input__attrs__class={'form-control': False},
     ),
     Action=dict(
         shortcuts=dict(
@@ -98,7 +99,6 @@ bootstrap_base = Style(
                 header__attrs__title='Select all',
                 header__attrs__class={'text-center': True},
                 cell__attrs__class={'text-center': True},
-
             ),
             number=dict(
                 cell__attrs__class={'text-right': True},
@@ -108,7 +108,7 @@ bootstrap_base = Style(
             delete=dict(
                 cell__link__attrs__class={'text-danger': True},
             ),
-        )
+        ),
     ),
     Query=dict(
         form__iommi_style='bootstrap_horizontal',
@@ -129,7 +129,7 @@ bootstrap_base = Style(
             'bg-primary': True,
         },
         items_container__attrs__class={'navbar-nav': True},
-        items_container__tag='ul'
+        items_container__tag='ul',
     ),
     MenuItem=dict(
         tag='li',
@@ -151,7 +151,12 @@ bootstrap_base = Style(
             'bg-primary': False,
             'navbar': False,
             'navbar-dark': False,
-        }
+        },
+        items_container__attrs__class={
+            'pl-0': True,
+            'mb-0': True,
+            'small': True,
+        },
     ),
     Admin=dict(
         parts__menu=dict(
@@ -169,6 +174,7 @@ bootstrap = Style(
 )
 bootstrap_horizontal = Style(
     bootstrap,
+    internal=True,
     Field=dict(
         shortcuts=dict(
             boolean__label__attrs__class={
