@@ -4,7 +4,7 @@ from iommi.asset import Asset
 
 water = Style(
     base,
-    assets__css=Asset.css(
+    root__assets__css=Asset.css(
         attrs=dict(
             href='https://cdn.jsdelivr.net/gh/kognise/water.css@latest/dist/dark.min.css',
         ),
